﻿using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;

namespace Falcon.Middleware
{
    /// <summary>
    /// 中间件基类
    /// </summary>
    public abstract class MiddlewareBase
    {
        private readonly RequestDelegate _next;

        /// <summary>
        /// 通过请求代理构造中间件
        /// </summary>
        /// <param name="next">下一个中间件</param>
        public MiddlewareBase(RequestDelegate next) {
            _next = next;
        }
        /// <summary>
        /// 异步执行中间件
        /// </summary>
        /// <param name="context">请求上下文</param>
        /// <returns></returns>
        public async Task InvokeAsync(HttpContext context) {
            InvokeNextBefore(context);
            await _next(context);
            InvokeNextAfter(context);
        }
        /// <summary>
        /// 在执行下一个中间件之前执行
        /// </summary>
        /// <param name="context">请求上下文</param>
        public virtual void InvokeNextBefore(HttpContext context) { }
        /// <summary>
        /// 在执行下一个中间件之后执行
        /// </summary>
        /// <param name="context">请求上下文</param>
        public virtual void InvokeNextAfter(HttpContext context) { }
    }
}
